@@ -3,6 +3,7 @@
 namespace PHPStan\PhpDocParser\Parser;
 
 use Iterator;
+use PHPStan\PhpDocParser\Ast\Attribute;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprArrayItemNode;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprArrayNode;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprFalseNode;
@@ -13,6 +14,7 @@ use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprNullNode;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprStringNode;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstExprTrueNode;
 use PHPStan\PhpDocParser\Ast\ConstExpr\ConstFetchNode;
+use PHPStan\PhpDocParser\Ast\NodeTraverser;
 use PHPStan\PhpDocParser\Lexer\Lexer;
 use PHPUnit\Framework\TestCase;
 
@@ -54,6 +56,38 @@ class ConstExprParserTest extends TestCase
 	}
 
 
+	/**
+	 * @dataProvider provideTrueNodeParseData
+	 * @dataProvider provideFalseNodeParseData
+	 * @dataProvider provideNullNodeParseData
+	 * @dataProvider provideIntegerNodeParseData
+	 * @dataProvider provideFloatNodeParseData
+	 * @dataProvider provideStringNodeParseData
+	 * @dataProvider provideArrayNodeParseData
+	 * @dataProvider provideFetchNodeParseData
+	 *
+	 * @dataProvider provideWithTrimStringsStringNodeParseData
+	 */
+	public function testVerifyAttributes(string $input): void
+	{
+		$tokens = new TokenIterator($this->lexer->tokenize($input));
+		$constExprParser = new ConstExprParser(true, true, [
+			'lines' => true,
+			'indexes' => true,
+		]);
+		$visitor = new NodeCollectingVisitor();
+		$traverser = new NodeTraverser([$visitor]);
+		$traverser->traverse([$constExprParser->parse($tokens)]);
+
+		foreach ($visitor->nodes as $node) {
+			$this->assertNotNull($node->getAttribute(Attribute::START_LINE), (string) $node);
+			$this->assertNotNull($node->getAttribute(Attribute::END_LINE), (string) $node);
+			$this->assertNotNull($node->getAttribute(Attribute::START_INDEX), (string) $node);
+			$this->assertNotNull($node->getAttribute(Attribute::END_INDEX), (string) $node);
+		}
+	}
+
+
 	public function provideTrueNodeParseData(): Iterator
 	{
 		yield [
@@ -118,6 +152,16 @@ class ConstExprParserTest extends TestCase
 			new ConstExprIntegerNode('123'),
 		];
 
+		yield [
+			'+123',
+			new ConstExprIntegerNode('+123'),
+		];
+
+		yield [
+			'-123',
+			new ConstExprIntegerNode('-123'),
+		];
+
 		yield [
 			'0b0110101',
 			new ConstExprIntegerNode('0b0110101'),
@@ -142,6 +186,26 @@ class ConstExprParserTest extends TestCase
 			'-0X7Fb4',
 			new ConstExprIntegerNode('-0X7Fb4'),
 		];
+
+		yield [
+			'123_456',
+			new ConstExprIntegerNode('123456'),
+		];
+
+		yield [
+			'0b01_01_01',
+			new ConstExprIntegerNode('0b010101'),
+		];
+
+		yield [
+			'-0X7_Fb_4',
+			new ConstExprIntegerNode('-0X7Fb4'),
+		];
+
+		yield [
+			'18_446_744_073_709_551_616', // 64-bit unsigned long + 1, larger than PHP_INT_MAX
+			new ConstExprIntegerNode('18446744073709551616'),
+		];
 	}
 
 
@@ -178,8 +242,13 @@ class ConstExprParserTest extends TestCase
 		];
 
 		yield [
-			'-123',
-			new ConstExprIntegerNode('-123'),
+			'+123.5',
+			new ConstExprFloatNode('+123.5'),
+		];
+
+		yield [
+			'-123.',
+			new ConstExprFloatNode('-123.'),
 		];
 
 		yield [
@@ -206,6 +275,36 @@ class ConstExprParserTest extends TestCase
 			'-12.3e-4',
 			new ConstExprFloatNode('-12.3e-4'),
 		];
+
+		yield [
+			'-1_2.3_4e5_6',
+			new ConstExprFloatNode('-12.34e56'),
+		];
+
+		yield [
+			'123.4e+8',
+			new ConstExprFloatNode('123.4e+8'),
+		];
+
+		yield [
+			'.4e+8',
+			new ConstExprFloatNode('.4e+8'),
+		];
+
+		yield [
+			'123E+80',
+			new ConstExprFloatNode('123E+80'),
+		];
+
+		yield [
+			'8.2023437675747321', // greater precision than 64-bit double
+			new ConstExprFloatNode('8.2023437675747321'),
+		];
+
+		yield [
+			'-0.0',
+			new ConstExprFloatNode('-0.0'),
+		];
 	}
 
 
